#include "oled_switch_hw.h"
#include "paint_bitmap.h"

#define IMG_SIZE      6144      //For still images this size is good, because display is only this size.
#define BIG_IMG_SIZE  12288     //For scrolling, you must use bigger image, because of firmware on module,
                                //( Module attempts to scroll 96x64 display despite memory allocated being smaller )
//GPIOs
sbit OLED_SWITCH_RST         at LATC1_bit;
sbit OLED_SWITCH_RST_DIR     at TRISC1_bit;
sbit OLED_SWITCH_CS          at LATC2_bit;
sbit OLED_SWITCH_CS_DIR      at TRISC2_bit;
sbit OLED_SWITCH_CMD         at LATD0_bit;
sbit OLED_SWITCH_CMD_DIR     at TRISD0_bit;

//Function Prototypes
void system_setup( buffer_size_t size );

void main()
{
    //Local Declarations
    scroll_t scroll;               //Used for scroll settings

    scroll.col_horiz_shift = 0x01; //Shift left to right
    scroll.row_offset      = 0;    //No offset of rows
    scroll.row_horiz_shift = 64;   //64: BIG IMAGE, 47:SMALL ( SMALL doesn't work well, because of firmware on click side )
    scroll.row_vert_shift  = 5;    //No vertical shifting
    scroll.interval        = 0;    //1: 10 frames between each shift

    //Setup
    system_setup( BIG ); //Provide size of allocated space for images ( 64x48 or 96x64 )

    oled_switch_deactivate_scrolling();                                   //Scrolling must be absolutely deactivated first
    oled_switch_scrolling_setup( scroll );                                //Next, setup scroll settings
    oled_switch_draw_565_img( oled_switch_logo_big_bmp, BIG_IMG_SIZE );                      //Draw the image to scroll
    oled_switch_activate_scrolling();                                     //Activate scrolling on screen

    while( 1 )//Forever
    {

    }
}

void system_setup( buffer_size_t size )
{

    //SPI Clock
    PLLFBD = 70;             // PLL multiplier M=70
    CLKDIV = 0x0000;         // PLL prescaler N1=2, PLL postscaler N2=2

    //UART remapping
    PPS_Mapping(100, _INPUT,  _U1RX);              // Sets pin RP100 to be Input, and maps U1RX to it
    PPS_Mapping(101, _OUTPUT, _U1TX);              // Sets pin RP101 to be Output, and maps U1TX to it
    Delay_ms(300);

    //GPIOs
    OLED_SWITCH_RST_DIR = 0;    //RST
    OLED_SWITCH_CS_DIR  = 0;    //CS
    OLED_SWITCH_CMD_DIR = 0;    //CMD or CD  ( Command / Data )

    //SPI Remapping
    PPS_Mapping(104, _OUTPUT,  _SDO3);             // Sets pin RP104 to be Output, and maps SD03 to it
    PPS_Mapping(98, _INPUT, _SDI3);                // Sets pin RP98 to be Input, and maps SD01 to it
    PPS_Mapping(79, _OUTPUT, _SCK3OUT);            // Sets pin RP79 to be Output, and maps SCK3 to it
    // Setting output frequency to 140MHz
    Delay_ms(300);

    // SPI
    SPI3_Init_Advanced(_SPI_MASTER,
                     _SPI_8_BIT,
                     _SPI_PRESCALE_SEC_8,
                     _SPI_PRESCALE_PRI_16,
                     _SPI_SS_DISABLE,
                     _SPI_DATA_SAMPLE_MIDDLE,
                     _SPI_CLK_IDLE_HIGH,
                     _SPI_ACTIVE_2_IDLE);
    Delay_ms( 500 );

    //UART
    UART1_Init(9600);                      // Initialize UART module at 9600 bps
    Delay_ms(300);                         // Wait for UART module to stabilize

    UART1_Write_Text( "UART Initialized\r\n" );
    UART1_Write_Text( "SPI Initialized\r\n" );

    //Initialize OLED Switch click
    oled_switch_init( size );
    Delay_ms( 300 );
    UART1_Write_Text( "OLED Switch click Initialized\r\n" );

}